#define DEBUG

#include <avr/io.h>

#define PIN_SIN_WAVE  3 /* PWM出力ピン（OC2B）  */

#define N_WAVE  32      /* 正弦波テーブルの数 */

unsigned int wave[N_WAVE];  // 正弦波テーブル
volatile int i_wave;        // 正弦波テーブルの参照ポインタ

void setup() {
  float unit_deg;
  int i;

  i_wave = 0;
 
  /* 正弦波テーブル作成 */
  unit_deg = (2.0 * 3.141592) / (float)(N_WAVE);	// 一周期は2πラジアン

  for (i = 0; i < N_WAVE; i++) {
     wave[i] = (unsigned int)((((sin(unit_deg * (float)i) + 1.0) / 2.0) * 255.0) + 0.5);
  }

#ifdef DEBUG
  Serial.begin(9600);

  for (i = 0; i < N_WAVE; i++) {
    Serial.print(i);
    Serial.print(" ");
    Serial.println(wave[i]);
  }
#endif


  /* 正弦波出力割込み周期設定 */
  TCCR1A = 0;             // CTC動作（比較一致クリア）
  TCCR1B = (1 << WGM12) | (1 << CS10);  // クロック分周なし
  OCR1A = 499;            // 割込みタイミング（0～499で500カウント）
  TIMSK1 = (1 << OCIE1A); // 割込み許可（比較A割込み）


  /* 正弦波用PWM周波数設定 */
  pinMode(PIN_SIN_WAVE, OUTPUT);

  TCCR2A = (1 << COM2B1) | (0 << COM2B0) | (1 << WGM21) | (1 << WGM20);
  TCCR2B = (1 << CS20);
}

/* 割込み: 正弦波出力 */
ISR (TIMER1_COMPA_vect) {
  if (i_wave >= N_WAVE) {
    i_wave = 0;
  }

  analogWrite(PIN_SIN_WAVE, wave[i_wave]);
  i_wave++;
}


void loop() {
}
